# Copyright (C) 2014 Canonical Ltd.
# Author: Michael Vogt <mvo@ubuntu.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import getpass
import json
import os
import sys

import oauthlib.oauth1

from click.repository import http_request


UBUNTUONE_API_BASE = "https://login.ubuntu.com/api/v2"
UBUNTUONE_OAUTH_API = UBUNTUONE_API_BASE + "/tokens/oauth"


def get_oauth_token(username, password, token_name):
    data = {"email": username,
            "password": password,
            "token_name": token_name,
    }
    data = json.dumps(data).encode("utf-8")
    resp, content = http_request(
        UBUNTUONE_OAUTH_API,
        data=data,
        headers={"content-type": "application/json"})
    print(resp)
    if resp.status != 200:
        raise Exception("get_oauth_token failed with status %s" % resp.status)
    token = json.loads(content.decode("utf-8"))
    return token


def get_oauth_headers_for_uri(uri, token):
    client = oauthlib.oauth1.Client(token["consumer_key"],
                                    token["consumer_secret"],
                                    token["token_key"],
                                    token["token_secret"])
    new_uri, headers, body = client.sign(uri)
    if new_uri != uri:
        raise Exception("uri change not supported")
    if body is not None:
        raise Exception("body != None not supported")
    return headers


if __name__ == "__main__":
    username = sys.argv[1]
    uri = sys.argv[2]

    TOKEN_FILE = "%s.token" % username
    if not os.path.exists(TOKEN_FILE):
        pw = getpass.getpass()
        token = get_oauth_token(username, pw, "foo-token")
        with open(TOKEN_FILE, "w") as f:
            f.write(json.dumps(token))

    with open(TOKEN_FILE) as f:
        token = json.loads(f.read())

    resp, content = http_request(
        uri, headers=get_oauth_headers_for_uri(uri, token))
    print(resp)
